// hardcodet.net Logging
// Copyright (c) 2009 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This library is free software; you can redistribute it and/or
// modify it in any way you see fit as long as this copyright
// notice is not being removed.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE




using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace Hardcodet.Util.Logging
{
  /// <summary>
  /// Encapsulates logging data that can be submitted to any
  /// given <see cref="ILogger"/> implementation.
  /// </summary>
  public class LogItem : ICloneable
  {

    #region properties

    /// <summary>
    /// A summarizing title for the logged entry. Defaults to
    /// <c>String.Empty</c>.
    /// </summary>
    public string Title { get; set; }


    /// <summary>
    /// The logged message body. Defaults to
    /// <c>String.Empty</c>.
    /// </summary>
    public string Message { get; set; }


    /// <summary>
    /// An optional error message, which is appended
    /// to the <see cref="Message"/> body. Defaults to
    /// <c>String.Empty</c>.
    /// </summary>
    public string ErrorMessage { get; set; }


    /// <summary>
    /// Category name which is used to group logged entries, or
    /// to log to different channels (implementation is up to
    /// the logging framework).
    /// </summary>
    public List<string> Categories { get; private set; }


    /// <summary>
    /// Importance of the logged entry. This value can be used for informational purpose
    /// only, or to filter messages depending on their priority (implementation is up
    /// to the logging framework).
    /// <para>
    /// Defaults to -1.
    /// </para>
    /// </summary>
    public int Priority { get; set; }


    /// <summary>
    /// Event number or identifier. Defaults to 0.
    /// </summary>
    public int EventId { get; set; }


    /// <summary>
    /// Log entry severity as a <see cref="Severity"/> enumeration.
    /// (Unspecified, Information, Warning or Error).
    /// Defaults to <see cref="System.Diagnostics.TraceEventType.Information"/>.
    /// </summary>
    public TraceEventType Severity { get; set; }


    /// <summary>
    /// Date and time of the log entry. If no explicietly
    /// set, this property is provides the timestamp of
    /// the object's creation.
    /// </summary>
    public DateTime TimeStamp { get; set; }

    #endregion


    /// <summary>
    /// Inits an new <see cref="LogItem"/> instance which
    /// is initialized with default values.
    /// </summary>
    public LogItem()
    {
      Title = String.Empty;
      Message = String.Empty;
      ErrorMessage = String.Empty;
      Severity = TraceEventType.Information;
      Priority = -1;

      TimeStamp = DateTime.Now;
      Categories = new List<string>();
    }



    #region clone

    /// <summary>
    /// Creates a new <see cref="LogItem"/> that is a copy of the current instance.
    /// </summary>
    /// <implements>ICloneable.Clone</implements>
    /// <returns>A new <c>LogItem</c> that is a copy of the current instance.</returns>
    public object Clone()
    {
      LogItem clone = new LogItem();

      clone.Title = Title;
      clone.Message = Message;
      clone.EventId = EventId;
      clone.Severity = Severity;
      clone.Priority = Priority;
      clone.TimeStamp = TimeStamp;

      // copy categories
      clone.Categories.AddRange(Categories);

      return clone;
    }

    #endregion


    /// <summary>
    /// Generates a formatted string-representation of the item.
    /// </summary>
    /// <returns>Formatted log message.</returns>
    public string ToLogMessage()
    {
      StringBuilder builder = new StringBuilder();

      //init entry
      builder.AppendLine();
      
      int titleLength = Math.Max((Title ?? "").Length, 20);
      builder.AppendLine("".PadRight(titleLength, '*'));

      if (!String.IsNullOrEmpty(Title))
      {
        builder.AppendLine(Title);
      }

      //write timestamp
      builder.AppendLine(TimeStamp.ToString("G"));


      //write categories, if any
      if (Categories.Count > 0)
      {
        for (int i = 0; i < Categories.Count; i++)
        {
          string cat = Categories[i];

          builder.Append(cat);
          if (i < Categories.Count - 1) builder.Append(", ");
        }

        builder.AppendLine();
      }

      //write logging severity
      builder.AppendLine(String.Format("Severity: {0}\n", Severity));

      //write IDs
      builder.AppendLine(String.Format("Event ID = {0}", EventId));
      builder.AppendLine(String.Format("Priority = {0}", Priority));

      //write message
      builder.AppendLine(Message);

      //write error message, if any
      if (!String.IsNullOrEmpty(ErrorMessage))
      {
        builder.AppendLine("\n--\n");
        builder.AppendLine(ErrorMessage);
      }

      //close entry
      builder.AppendLine("".PadRight(titleLength, '*'));
      builder.AppendLine("");

      return builder.ToString();
    }
  }
}