// hardcodet.net Logging
// Copyright (c) 2009 Philipp Sumi
// Contact and Information: http://www.hardcodet.net
//
// This library is free software; you can redistribute it and/or
// modify it in any way you see fit as long as this copyright
// notice is not being removed.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE



using System;
using System.Diagnostics;

namespace Hardcodet.Util.Logging
{
  
  /// <summary>
  /// Abstract base class which provides most logging method
  /// overloads for an <see cref="ILogger"/> implementation.
  /// </summary>
  public abstract class LoggerBase : ILogger
  {
    /// <summary>
    /// Logs an informational message (<see cref="TraceEventType.Information"/>).
    /// </summary>
    /// <param name="message">The message to be logged.</param>
    public virtual void Log(string message)
    {
      //call overload
      Log(message, TraceEventType.Information);
    }


    /// <summary>
    /// Logs an exception with a logging level of
    /// <see cref="TraceEventType.Error"/>.
    /// </summary>
    /// <param name="exception">Exception to be logged.</param>
    public virtual void Log(Exception exception)
    {
      //call overload
      Log(exception.ToString(), TraceEventType.Error);
    }


    /// <summary>
    /// Logs an error with an additional message with a logging level of
    /// <see cref="TraceEventType.Error"/>.
    /// </summary>
    /// <param name="message">Additional information regarding the
    /// logged exception.</param>
    /// <param name="exception">Exception to be logged.</param>
    public virtual void Log(string message, Exception exception)
    {
      //call overload
      Log(message, exception, TraceEventType.Error);
    }


    /// <summary>
    /// Logs a message with a given logging severity.
    /// </summary>
    /// <param name="message">The message to be logged.</param>
    /// <param name="severity">Logging category.</param>
    public virtual void Log(string message, TraceEventType severity)
    {
      LogItem item = new LogItem();
      item.Message = message;
      item.Severity = severity;
      
      Log(item);
    }


    /// <summary>
    /// Logs an exception with a given logging severity and an
    /// additional message.
    /// </summary>
    /// <param name="message">Additional information regarding the
    /// logged exception.</param>
    /// <param name="exception">Exception to be logged.</param>
    /// <param name="severity">Logging category.</param>
    public virtual void Log(string message, Exception exception, TraceEventType severity)
    {     
      LogItem item = new LogItem();
      item.Message = message;
      item.Severity = severity;

      if (exception != null)
        item.ErrorMessage = exception.ToString();
      else
        item.ErrorMessage = "[No error information available]";
        
      Log(item);
    }


    /// <summary>
    /// Creates a new log entry based on a given log item.
    /// </summary>
    /// <param name="item">Encaspulates logging information.</param>
    /// <exception cref="ArgumentNullException">If <paramref name="item"/>
    /// is a null reference.</exception>
    public abstract void Log(LogItem item);
  }
}